import numpy as np
from pyqtgraph.graphicsItems.ImageItem import ImageItem
from pyqtgraph.graphicsItems.ViewBox import ViewBox
import pyqtgraph as pg
from pyqtgraph.colormap import ColorMap

import os
from numpy import nanmax, nanmin

if "QT_PREFERRED_BINDING" not in os.environ:
    os.environ["QT_PREFERRED_BINDING"] = os.pathsep.join(
        #["PySide2", "PyQt5", "PySide", "PyQt4"]
        ["PyQt5", "PySide2", "PySide", "PyQt4"]
    )

import Qt
from Qt import QtCore
from Qt import QtWidgets
from Qt import QtGui


"""
ImageShowBasicWidget.py -  Widget for basic image dispay and analysis
Copyright 2010  Luke Campagnola
Distributed under MIT/X11 license. See license.txt for more infomation.

Widget used for displaying 2D or 3D data. Features:
  - float or int (including 16-bit int) image display via ImageItem
  - zoom/pan via GraphicsView
  - black/white level controls
  - time slider for 3D data sets
  - ROI plotting
  - Image normalization through a variety of methods
"""


class ColorMaps(object):

    BGYR= 'BGYR'
    BLUE= 'BLUE'
    GREY= 'GREY'
    INV_BLUE= 'INV_BLUE'
    INV_RED= 'INV_RED'
    RED= 'RED'
    SATURATION= 'SATURATION'

    def __init__(self):
        self._createColorMaps()

    def _createColorMapSaturation(self):
        pos = np.array([0.0, 0.1, 0.5, 0.9, 1.0])
        color = np.array([[0, 0, 255, 128],
                          [255, 255, 255, 0],
                          [255, 255, 255, 0],
                          [255, 255, 255, 0],
                          [255, 0, 0, 128]], dtype=np.ubyte)
        cmap = pg.ColorMap(pos, color, ColorMap.RGB)
        lut = cmap.getLookupTable(0.0, 1.0, 128)
        return lut


    def _createColorMapBGYR(self):
        pos = np.array([0.0, 0.33, 0.66, 1.0])
        color = np.array([[0, 0, 255, 255],
                          [0, 255, 0, 255],
                          [255, 255, 0, 255],
                          [255, 0, 0, 255]], dtype=np.ubyte)
        cmap = pg.ColorMap(pos, color, ColorMap.RGB)
        lut = cmap.getLookupTable(0.0, 1.0, 256)
        return lut


    def _createColorMapRed(self):
        pos = np.array([0.0, 1.0])
        color = np.array([[0, 0, 0, 255],
                          [255, 64, 0, 255]], dtype=np.ubyte)
        cmap = pg.ColorMap(pos, color, ColorMap.RGB)
        lut = cmap.getLookupTable(0.0, 1.0, 256)
        return lut


    def _createColorMapInvRed(self):
        pos = np.array([0.0, 1.0])
        color = np.array([[255, 64, 0, 64],
                          [255, 255, 255, 64]], dtype=np.ubyte)
        cmap = pg.ColorMap(pos, color, ColorMap.RGB)
        lut = cmap.getLookupTable(0.0, 1.0, 256)
        return lut


    def _createColorMapBlue(self):
        pos = np.array([0.0, 1.0])
        color = np.array([[0, 0, 0, 255],
                          [0, 64, 255, 255]], dtype=np.ubyte)
        cmap = pg.ColorMap(pos, color, ColorMap.RGB)
        lut = cmap.getLookupTable(0.0, 1.0, 256)
        return lut


    def _createColorMapInvBlue(self):
        pos = np.array([0.0, 1.0])
        color = np.array([[0, 64, 255, 64],
                          [255, 255, 255, 64]], dtype=np.ubyte)
        cmap = pg.ColorMap(pos, color, ColorMap.RGB)
        lut = cmap.getLookupTable(0.0, 1.0, 256)
        return lut


    def _createColorMapGrey(self):
        pos = np.array([0.0, 1.0])
        color = np.array([[0, 0, 0, 255],
                          [255, 255, 255, 255]], dtype=np.ubyte)
        cmap = pg.ColorMap(pos, color, ColorMap.RGB)
        lut = cmap.getLookupTable(0.0, 1.0, 256)
        return lut


    def _createColorMaps(self):
        self._maps={}
        self._maps[self.BGYR]= self._createColorMapBGYR()
        self._maps[self.BLUE]= self._createColorMapBlue()
        self._maps[self.GREY]= self._createColorMapGrey()
        self._maps[self.INV_BLUE]= self._createColorMapInvBlue()
        self._maps[self.INV_RED]= self._createColorMapInvRed()
        self._maps[self.RED]= self._createColorMapRed()
        self._maps[self.SATURATION]= self._createColorMapSaturation()


    def get(self, mapName):
        return self._maps[mapName]



class ImageShowBasicWidget(QtWidgets.QWidget):

    def __init__(self,
                 parent=None,
                 name="ImageShowWidget",
                 view=None,
                 imageItem=None,
                 *args):
        super(ImageShowBasicWidget, self).__init__()

        self.name = name
        self.image = None
        self.axes = {}
        self.imageDisp = None
        self._useAutoLevels= True
        self._cursorx= 0
        self._cursory= 0

        from .image_show_basic_widget_ui import Ui_Form
        self.ui = Ui_Form()
        self.ui.setupUi(self)

        self.scene = self.ui.graphicsView.scene()

        if view is None:
            self.view = ViewBox()
        else:
            self.view = view
        self.ui.graphicsView.setCentralItem(self.view)
        self.view.setAspectLocked(True)
        #self.view.invertY()

        if imageItem is None:
            self.imageItem = ImageItem()
        else:
            self.imageItem = imageItem
        self.view.addItem(self.imageItem)
        self.imageItem.setBorder('k')
        self.imageItem.setLevels([0, 255])

        ## wrap functions from view box
        for fn in ['addItem', 'removeItem']:
            setattr(self, fn, getattr(self.view, fn))

        self._colormaps= ColorMaps()
        self.setColormap(ColorMaps.GREY)

        #self._addOverlay()
        self._setupSaturarationOverlay()
        self._setupMouseMovedSignalProxy()

        self.view.register(self.name)



    def _onMouseMove(self, ev):
        mousePoint = self.view.mapSceneToView(ev[0])
        try:
            szX, szY = self.imageDisp.shape
            self._cursorx= int(np.clip(mousePoint.x(), 0, szX - 1))
            self._cursory= int(np.clip(mousePoint.y(), 0, szY - 1))
            self._updateLabelInfo()
        except Exception:
            pass

    def _updateLabelInfo(self):
        value= self.imageDisp[self._cursorx, self._cursory]
        self.ui.labelInfo.setText(
            "X:%4d Y:%4d I:  %g" % (
                self._cursorx, self._cursory, value))


    def _setupMouseMovedSignalProxy(self):
        self._mouseMoveProxy= pg.SignalProxy(
            self.scene.sigMouseMoved,
            rateLimit=20,
            slot=self._onMouseMove)

    def _setupSaturarationOverlay(self):
        self._wantsSaturationOverlay= False
        self._maxSaturation= np.inf
        self._minSaturation= -np.inf
        self._saturationOverlayImageItem= ImageItem()
        self._saturationOverlayImage= None
        self.view.addItem(self._saturationOverlayImageItem)
        self._saturationOverlayImageItem.setCompositionMode(
            QtGui.QPainter.CompositionMode_SourceOver)
        self._saturationOverlayImageItem.setLookupTable(
            self._colormaps.get(ColorMaps.SATURATION))


#     def _addOverlay(self):
#         self.overlayImageItem = ImageItem()
#         self.view.addItem(self.overlayImageItem)
#         self.overlayImageItem.setCompositionMode(
#             #QtGui.QPainter.CompositionMode_Plus)
#             QtGui.QPainter.CompositionMode_SourceOver)
#             #QtGui.QPainter.CompositionMode_Overlay)
#         overlayImage= np.identity(100)
#         overlayImage[25:75, 25:75]=1
#         self.overlayImageItem.setImage(overlayImage)
#         self.overlayImageItem.setLookupTable(
#                 self._colormaps.get(ColorMaps.INV_BLUE))


    def keyPressEvent(self, event):
        key = event.key()
        if key == QtCore.Qt.Key_C:
            print("event key %s: rollColorMap" % str(key))
            self._rollColorMap()
        if key == QtCore.Qt.Key_L:
            self._useAutoLevels= not self._useAutoLevels
            print("event key %s: toggle autolevels %s" % (
                str(key), self._useAutoLevels))
        if key == QtCore.Qt.Key_O:
            self._optimizeLevels()
            print("event key %s: optimize levels " % str(key))
        if key == QtCore.Qt.Key_Q:
            self._wantsSaturationOverlay= not \
                self._wantsSaturationOverlay
            self._updateImage()
        super(ImageShowBasicWidget, self).keyPressEvent(event)


    def mouseClickEvent(self, ev):
        if ev.button() == QtCore.Qt.LeftButton:
            ev.accept()
            print("click on %s" % ev.pos())


    def setColormap(self, colorMapName):
        self.imageItem.setLookupTable(
            self._colormaps.get(colorMapName))
        self._currentColorMapName= colorMapName


    def _rollColorMap(self):
        if self._currentColorMapName == ColorMaps.BGYR:
            self.setColormap(ColorMaps.RED)
        elif self._currentColorMapName == ColorMaps.RED:
            self.setColormap(ColorMaps.BLUE)
        elif self._currentColorMapName == ColorMaps.BLUE:
            self.setColormap(ColorMaps.GREY)
        elif self._currentColorMapName == ColorMaps.GREY:
            self.setColormap(ColorMaps.BGYR)


    def _optimizeLevels(self):
        self._useAutoLevels= False
        self.setLevels(self.levelMin, self.levelMax)
        self._updateImage()


    def setBackgroundColor(self, color):
        self.view.setBackgroundColor(color)


    def setOverlayImage(self, overlayImage):
        self.overlayImageItem.setImage(overlayImage)


    def setImageSaturationValues(self,
                                 minSaturation,
                                 maxSaturation):
        self._minSaturation= minSaturation
        self._maxSaturation= maxSaturation


    def enableSaturationOverlay(self, trueOrFalse):
        self._wantsSaturationOverlay= trueOrFalse


    def _showSaturationOverlay(self):
        tooMuch= np.ones(self.imageDisp.shape, dtype=np.ubyte) * 127
        if self._wantsSaturationOverlay:
            tooMuch[self.imageDisp > self._maxSaturation]= 255
            tooMuch[self.imageDisp < self._minSaturation]= 0
            if not np.array_equal(tooMuch, self._saturationOverlayImage):
                self._saturationOverlayImageItem.setImage(
                    tooMuch, levels=[0, 255])
                self._saturationOverlayImage= tooMuch
        else:
            if self._saturationOverlayImage is not None:
                self._saturationOverlayImageItem.clear()
                self._saturationOverlayImage= None


    def setImage(self,
                 img,
                 autoRange=True,
                 useAutoLevels=None,
                 levels=None,
                 axes=None,
                 pos=None,
                 scale=None,
                 transform=None,
                 ):
        """
        Set the image to be displayed in the widget.

        ================== ===========================================================================
        **Arguments:**
        img                (numpy array) the image to be displayed. See :func:`ImageItem.setImage` and
                           *notes* below.
        xvals              (numpy array) 1D array of z-axis values corresponding to the third axis
                           in a 3D image. For video, this array should contain the time of each frame.
        autoRange          (bool) whether to scale/pan the view to fit the image.
        useAutoLevels      (bool) whether to update the white/black levels to fit the image.
        levels             (min, max); the white and black level values to use.
        axes               Dictionary indicating the interpretation for each axis.
                           This is only needed to override the default guess. Format is::

                               {'t':0, 'x':1, 'y':2, 'c':3};

        pos                Change the position of the displayed image
        scale              Change the scale of the displayed image
        transform          Set the transform of the displayed image. This option overrides *pos*
                           and *scale*.
        autoHistogramRange If True, the histogram y-range is automatically scaled to fit the
                           image data.
        ================== ===========================================================================

        **Notes:**

        For backward compatibility, image data is assumed to be in column-major order (column, row).
        However, most image data is stored in row-major order (row, column) and will need to be
        transposed before calling setImage()::

            imageview.setImage(imagedata.T)

        This requirement can be changed by the ``imageAxisOrder``
        :ref:`global configuration option <apiref_config>`.

        """

        if hasattr(img, 'implements') and img.implements('MetaArray'):
            img = img.asarray()

        if not isinstance(img, np.ndarray):
            required = ['dtype', 'max', 'min', 'ndim', 'shape', 'size']
            if not all([hasattr(img, attr) for attr in required]):
                raise TypeError("Image must be NumPy array or any object "
                                "that provides compatible attributes/methods:\n"
                                "  %s" % str(required))

        self.image = img
        self.imageDisp = None


        if axes is None:
            x, y = (0, 1) if self.imageItem.axisOrder == 'col-major' else (1, 0)

            if img.ndim == 2:
                self.axes = {'t': None, 'x': x, 'y': y, 'c': None}
            elif img.ndim == 3:
                # Ambiguous case; make a guess
                if img.shape[2] <= 4:
                    self.axes = {'t': None, 'x': x, 'y': y, 'c': 2}
                else:
                    self.axes = {'t': 0, 'x': x+ 1, 'y': y+ 1, 'c': None}
            elif img.ndim == 4:
                # Even more ambiguous; just assume the default
                self.axes = {'t': 0, 'x': x+ 1, 'y': y+ 1, 'c': 3}
            else:
                raise Exception(
                    "Can not interpret image with dimensions %s" %
                    (str(img.shape)))
        elif isinstance(axes, dict):
            self.axes = axes.copy()
        elif isinstance(axes, list) or isinstance(axes, tuple):
            self.axes = {}
            for i in range(len(axes)):
                self.axes[axes[i]] = i
        else:
            raise Exception(
                "Can not interpret axis specification %s. "
                "Must be like {'t': 2, 'x': 0, 'y': 1} or "
                "('t', 'x', 'y', 'c')" % (str(axes)))

        for x in ['t', 'x', 'y', 'c']:
            self.axes[x] = self.axes.get(x, None)
        axes = self.axes


        self.currentIndex = 0

        if levels is None and useAutoLevels:
            self._useAutoLevels= useAutoLevels
        if levels is not None:  # this does nothing since getProcessedImage sets these values again.
            self.setLevels(*levels)

        self._updateImage()
        self._updateLabelInfo()

        self.imageItem.resetTransform()
        if scale is not None:
            self.imageItem.scale(*scale)
        if pos is not None:
            self.imageItem.setPos(*pos)
        if transform is not None:
            self.imageItem.setTransform(transform)


    def setLevels(self, min, max):
        """Set the min/max (bright and dark) levels."""
        self.imageItem.setLevels([min, max])


    def _updateImage(self):
        # Redraw image on screen
        if self.image is None:
            return

        image = self._getProcessedImage()

        # Transpose image into order expected by ImageItem
        if self.imageItem.axisOrder == 'col-major':
            axorder = ['t', 'x', 'y', 'c']
        else:
            axorder = ['t', 'y', 'x', 'c']
        axorder = [self.axes[ax] for ax in axorder if self.axes[ax] is not None]
        image = image.transpose(axorder)

        self.imageItem.setImage(image,
                                autoLevels=self._useAutoLevels)
        self._showSaturationOverlay()


    def _getProcessedImage(self):
        """Returns the image data after it has been processed by any normalization options in use.
        This method also sets the attributes self.levelMin and self.levelMax 
        to indicate the range of data in the image."""
        if self.imageDisp is None:
            self.imageDisp = self.image
            self.levelMin, self.levelMax = self._quickLevels(
                self.imageDisp)
            #list( map(float, self._quickLevels(self.imageDisp)))
        return self.imageDisp


    def _quickLevels(self, data):
        """
        Estimate the min/max values of *data* by subsampling.
        """
        while data.size > 1e6:
            ax = np.argmax(data.shape)
            sl = [slice(None)] * data.ndim
            sl[ax] = slice(None, None, 2)
            data = data[sl]
        return self._levelsFromMedianAndStd(data)


    def _levelsFromMinMax(self, data):
        return nanmin(data), nanmax(data)


    def _levelsFrom2And98Percentile(self, data):
        a= np.percentile(data, [3, 97], interpolation='lower')
        return a[0], a[1]


    def _levelsFromMedianAndStd(self, data):
        std= np.std(data)
        median= np.median(data)
        vmin = median - 3 * std
        vmax = median + 3 * std + 1
        return vmin, vmax


    def getView(self):
        """Return the ViewBox (or other compatible object) which displays the ImageItem"""
        return self.view


    def getImageItem(self):
        """Return the ImageItem for this ImageShowWidget."""
        return self.imageItem


    def close(self):
        """Closes the widget nicely, making sure to clear the graphics scene and release memory."""
        self.ui.graphicsView.close()
        self.scene.clear()
        del self.image
        del self.imageDisp
        super(ImageShowBasicWidget, self).close()
        self.setParent(None)





import os
import pyfits
import numpy as np

from plico.utils.decorator import override, returnsNone, returns, cacheResult
from pysilico.calibration.abstract_calibration_manager import \
    AbstractCalibrationManager
from pysilico.types.camera_frame import CameraFrame
from plico.utils.snapshotable import Snapshotable
from plico.utils.fits_file_based_calibration_manager import \
    FitsFileBasedCalibrationManager


class CalibrationManagerException(Exception):
    pass


class CalibrationManager(AbstractCalibrationManager,
                         FitsFileBasedCalibrationManager):

    def __init__(self, calibrationRootDir):
        self._calibRootDir = calibrationRootDir


    def _checkTag(self, tag, tagPurpose="unspecified"):
        if tag is None:
            raise CalibrationManagerException(
                "A tag (%s) must be given but is None" % tagPurpose)
        if len(tag) == 0:
            raise CalibrationManagerException(
                "A tag name (%s) must be valid but it is '%s'" % (
                    tagPurpose, tag))



    def getDarkFrameFileName(self, tag):
        return os.path.join(self._calibRootDir,
                            "dark_frame", "%s.fits" % tag)

    @override
    @returnsNone
    def saveDarkFrame(
            self, tag, darkCameraFrame, fitsHeader=None):
        self._checkTag(tag, "DarkCameraFrame")
        assert isinstance(darkCameraFrame, CameraFrame)

        fileName= self.getDarkFrameFileName(tag)
        self._createFoldersIfMissing(fileName)
        pyfits.writeto(fileName, darkCameraFrame.toNumpyArray(),
                       fitsHeader,
                       clobber=False)
        pyfits.append(fileName, np.array([darkCameraFrame.counter()]))


    @override
    @returns(CameraFrame)
    @cacheResult
    def loadDarkFrame(self, tag):
        self._checkTag(tag, "DarkCameraFrame")
        fileName= self.getDarkFrameFileName(tag)
        hdr= pyfits.getheader(fileName)
        hduList= pyfits.open(fileName)
        rawArray= hduList[0].data
        counter= hduList[1].data[0]
        try:
            snapshot= Snapshotable.fromFITSHeader(hdr)
        except Exception:
            snapshot= dict()
        darkCameraFrame= CameraFrame(rawArray, counter)
        return darkCameraFrame




from plico.utils.decorator import override, returns, returnsNone
from pysilico.calibration.abstract_calibration_manager import \
    AbstractCalibrationManager
from pysilico.types.camera_frame import CameraFrame



class InMemoryCalibrationManager(AbstractCalibrationManager):

    def __init__(self):
        self._darkFrameDict= {}


    @override
    @returnsNone
    def saveDarkFrame(
            self, tag, darkCameraFrame):
        assert isinstance(darkCameraFrame, CameraFrame)
        self._darkFrameDict[tag]= darkCameraFrame


    @override
    @returns(CameraFrame)
    def loadDarkFrame(self, tag):
        return self._darkFrameDict[str(tag)]



import abc
from six import with_metaclass
from plico.utils.decorator import returnsNone, returns
from pysilico.types.camera_frame import CameraFrame


class AbstractCalibrationManager(with_metaclass(abc.ABCMeta, object)):


    @abc.abstractmethod
    @returnsNone
    def saveDarkFrame(self, tag, darkCameraFrame):
        assert False


    @abc.abstractmethod
    @returns(CameraFrame)
    def loadDarkFrame(self, tag):
        assert False



from pysilico.utils.constants import Constants


def _getDefaultConfigFilePath():
    from plico.utils.config_file_manager import ConfigFileManager
    cfgFileMgr= ConfigFileManager(Constants.APP_NAME,
                                  Constants.APP_AUTHOR,
                                  Constants.THIS_PACKAGE)
    return cfgFileMgr.getConfigFilePath()


defaultConfigFilePath= _getDefaultConfigFilePath()


def camera(hostname, port):

    from pysilico.client.camera_client import CameraClient
    from plico.rpc.zmq_remote_procedure_call import ZmqRemoteProcedureCall
    from plico.rpc.zmq_ports import ZmqPorts
    from plico.rpc.sockets import Sockets


    rpc= ZmqRemoteProcedureCall()
    zmqPorts= ZmqPorts(hostname, port)
    sockets= Sockets(zmqPorts, rpc)
    return CameraClient(rpc, sockets)



import numpy as np


class CameraFrame(object):

    def __init__(self,
                 rawArray,
                 counter=0):
        self._array= rawArray.astype(np.float)
        self._counter = counter


    def __eq__(self, o):
        if self._counter != o.counter():
            return False
        if not np.array_equal(self._array, o.toNumpyArray()):
            return False
        return True


    def __ne__(self, o):
        return not self.__eq__(o)


    def __str__(self):
        return "frame-%d" % self._counter


    @staticmethod
    def fromNumpyArray(frameAsNumpyArray,
                       counter=0):
        return CameraFrame(
            frameAsNumpyArray,
            counter)


    def toNumpyArray(self):
        return self._array


    def counter(self):
        return self._counter


    def setCounter(self, counter):
        self._counter = counter



#!/usr/bin/env python
from pysilico.client.abstract_camera_client import AbstractCameraClient,\
    MultipleFrameRetriever, SnapshotEntry
from plico.rpc.abstract_remote_procedure_call import \
    AbstractRemoteProcedureCall
from plico.utils.logger import Logger
from plico.utils.decorator import override, returns, returnsForExample
from pysilico.types.camera_status import CameraStatus
from pysilico.utils.timeout import Timeout
from pysilico.types.camera_frame import CameraFrame
from plico.utils.snapshotable import Snapshotable
from plico.client.serverinfo_client import ServerInfoClient
from plico.client.hackerable_client import HackerableClient



class CameraClient(AbstractCameraClient,
                   HackerableClient,
                   ServerInfoClient):

    def __init__(self,
                 rpcHandler,
                 sockets):
        assert isinstance(rpcHandler, AbstractRemoteProcedureCall)

        self._rpcHandler= rpcHandler
        self._subscriberSocket= sockets.serverSubscriber()
        self._subscriberSocketAddress= sockets.serverSubscriberAddress()
        self._subscriberSocket.disconnect(self._subscriberSocketAddress)
        self._displaySocket= sockets.serverDisplay()
        self._displaySocketAddress= sockets.serverDisplayAddress()
        self._requestSocket= sockets.serverRequest()
        self._statusSocket= sockets.serverStatus()
        self._logger= Logger.of('Camera client')
        HackerableClient.__init__(self,
                                  self._rpcHandler,
                                  self._requestSocket,
                                  self._logger)
        ServerInfoClient.__init__(self,
                                  self._rpcHandler,
                                  self._requestSocket,
                                  self._logger)
        self._shape= None
        self._dtype= None
        self._lastFrame= None


    @override
    @returns(CameraStatus)
    def getStatus(
            self, timeoutInSec=Timeout.GET_FRAME_FOR_DISPLAY_TIMEOUT):
        return self._rpcHandler.receivePickable(
            self._statusSocket,
            timeoutInSec)


    @override
    @returns(CameraFrame)
    def getFrameForDisplay(
            self, timeoutInSec=Timeout.GET_FRAME_FOR_DISPLAY_TIMEOUT):
        return self._rpcHandler.recvCameraFrame(
            self._displaySocket,
            timeoutInSec=timeoutInSec)


    def _getLastAvailableFrame(self):
        self._lastFrame= self._rpcHandler.recvCameraFrame(
            self._subscriberSocket,
            timeoutInSec=Timeout.GET_FRAME_TIMEOUT)
        return self._lastFrame



    @override
    @returns(CameraFrame)
    def getFutureFrames(self,
                        numberOfReturnedImages,
                        numberOfFramesToAverageForEachImage=1,
                        timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        self._subscriberSocket.connect(self._subscriberSocketAddress)
        trashFirst= self._getLastAvailableFrame()
        frames= MultipleFrameRetriever.getFrames(
            numberOfReturnedImages,
            numberOfFramesToAverageForEachImage,
            self._getLastAvailableFrame)
        self._subscriberSocket.disconnect(self._subscriberSocketAddress)
        return frames


    @override
    @returnsForExample((1024, 1024))
    def shape(self):
        status= self.getStatus()
        return (status.frameHeight, status.frameWidth)


    def dtype(self):
        return self.getStatus().dtype


    def subSocket(self):
        return self._subscriberSocket


    @override
    def setExposureTime(self,
                        exposureTimeInMilliSeconds,
                        timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        self._logger.notice("Setting exposure time to %f ms" %
                            exposureTimeInMilliSeconds)
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'setExposureTime',
            [exposureTimeInMilliSeconds],
            timeout=timeoutSec)


    @override
    @returns(float)
    def exposureTime(self):
        return float(self.getStatus().exposureTimeInMilliSec)


    @override
    def getSnapshot(self,
                    prefix,
                    timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        status= self.getStatus(timeoutInSec=timeoutSec)
        self._logger.notice("Getting snapshot for %s " % prefix)
        return self._createSnapshotFromStatus(prefix, status)


    def _createSnapshotFromStatus(self, prefix, status):
        assert isinstance(status, CameraStatus)
        snapshot= {}
        snapshot[SnapshotEntry.CAMERA_NAME]= status.name
        snapshot[SnapshotEntry.EXPOSURE_TIME_MS]= status.exposureTimeInMilliSec
        snapshot[SnapshotEntry.FRAME_WIDTH_PX]= status.frameWidth
        snapshot[SnapshotEntry.FRAME_HEIGHT_PX]= status.frameHeight
        snapshot[SnapshotEntry.BINNING]= status.binning
        return Snapshotable.prepend(prefix, snapshot)



    @override
    def setBinning(self,
                   binning,
                   timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        self._logger.notice("Setting binning to %d " % binning)
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'setBinning',
            [binning],
            timeout=timeoutSec)


    @override
    @returns(int)
    def getBinning(self):
        return int(self.getStatus().binning)


    @override
    def setFrameRate(self, frameRate,
                     timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        self._logger.notice("Setting frameRate to %f" % frameRate)
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'setFrameRate',
            [frameRate],
            timeout=timeoutSec)


    @override
    @returns(float)
    def getFrameRate(self):
        return float(self.getStatus().frameRate)


    @override
    @returns(CameraFrame)
    def getDarkFrame(self, timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'getDarkFrame',
            [],
            timeout=timeoutSec)


    @override
    def setDarkFrame(self, darkFrame,
                     timeoutSec=Timeout.GET_FRAME_TIMEOUT):
        assert isinstance(darkFrame, CameraFrame)
        return self._rpcHandler.sendRequest(
            self._requestSocket, 'setDarkFrame',
            [darkFrame],
            timeout=timeoutSec)



#!/usr/bin/env python
import sys
from pysilico.gui.camera_control_gui import Runner


__version__ = "$Id: tipico_mirror_controller_1.py 31 2018-01-27 10:47:29Z lbusoni $"



def main():
    runner= Runner()
    print("%s" % sys.argv)
    sys.exit(runner.run(sys.argv[1:]))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import os
import sys
from shutil import rmtree

from setuptools import setup, Command

NAME = 'pysilico'
DESCRIPTION = 'Allied Vision AVT-Prosilica camera controller with PLICO'
URL = 'https://github.com/lbusoni/pysilico'
EMAIL = 'lorenzo.busoni@inaf.it'
AUTHOR = 'Lorenzo Busoni'
LICENSE= 'MIT'
KEYWORDS= 'plico, prosilica, avt, camera, laboratory, instrumentation control',


here = os.path.abspath(os.path.dirname(__file__))
# Load the package's __version__.py module as a dictionary.
about = {}
with open(os.path.join(here, NAME, '__version__.py')) as f:
    exec(f.read(), about)



class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git push --tags')

        sys.exit()


setup(name=NAME,
      description=DESCRIPTION,
      version=about['__version__'],
      classifiers=['Development Status :: 4 - Beta',
                   'Operating System :: POSIX :: Linux',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6',
                   ],
      long_description=open('README.md').read(),
      url=URL,
      author_email=EMAIL,
      author=AUTHOR,
      license=LICENSE,
      keywords=KEYWORDS,
      packages=['pysilico',
                'pysilico.calibration',
                'pysilico.client',
                'pysilico.gui',
                'pysilico.gui.image_show_widget',
                'pysilico.types',
                'pysilico.utils',
                ],
      entry_points={
          'gui_scripts': [
              'pysilico_gui=pysilico.gui.pysilico_gui:main',
          ],
      },
      package_data={
          'pysilico': ['conf/pysilico.conf'],
      },
      install_requires=["plico>=0.15",
                        "numpy",
                        "psutil",
                        "configparser",
                        "six",
                        "appdirs",
                        "pyfits",
                        "futures",
                        "rebin",
                        "pyqtgraph",
                        "pyside2",
                        "Qt.py",
                        "pathlib2; python_version < '3'"
                        ],
      include_package_data=True,
      test_suite='test',
      cmdclass={'upload': UploadCommand, },
      )
